@@ -1,5 +1,5 @@
 //
-// $Id: webmagick.js,v 1.22 1999/10/17 05:32:20 bfriesen Exp $
+// $Id: webmagick.js,v 1.37 2001/07/21 23:03:53 clindell Exp $
 //
 // JavaScript routines to display a set of images
 // Part of the WebMagick package
@@ -33,10 +33,74 @@ if( parent.innerWidth &&  parent.innerHeight) {
 // Define window names based on frame style option
 var frameTargets = new Object();
 
+var origFramestyle = htmlOpts["framestyle"];
+var origColumns = htmlOpts["columns"];
+var origRows = htmlOpts["rows"];
+var today = new Date();
+var expires_date = new Date(today.getYear() + 1,today.getMonth(),today.getDate(),0,0,0);
+
+// for testing
+//Set_Cookie('framestyle', '2', expires_date);
+//Set_Cookie('columns', '2', expires_date);
+//Set_Cookie('rows', '3', expires_date);
+
+// Should check to see if all of these are needed
+if (htmlOpts["config"] && htmlOpts["config"] != 0 && htmlOpts["tables"]) {
+	if (Get_Cookie('framestyle') != null) {
+		htmlOpts["framestyle"] = Get_Cookie('framestyle');
+		if (htmlOpts["framestyle"] != origFramestyle) {
+			// reset the cookie to increase the expiration	
+			// should do something about the path, but I'm not sure what - one of our current options?
+			Set_Cookie('frameStyle', htmlOpts["framestyle"], expires_date);	
+		}
+	}
+	if (Get_Cookie('rows') != null) {
+		htmlOpts["rows"] = Get_Cookie('rows');
+		if (htmlOpts["rows"] != origRows) {		
+			Set_Cookie('rows', htmlOpts["rows"], expires_date);	
+		}
+	}
+	if (Get_Cookie('columns') != null) {	
+		htmlOpts["columns"] = Get_Cookie('columns');
+		if (htmlOpts["columns"] != origColumns) {
+			Set_Cookie('columns', htmlOpts["columns"], expires_date);	
+		}
+	}
+	
+	if ((htmlOpts["rows"] != origRows) || (htmlOpts["columns"] != origColumns)) {		
+		var totalCount = 0;
+		for (i = 0; i < imageNames.length; i++) {
+			for (j = 0; j < imageNames[i].length; j++) {
+				totalCount++;			
+			}		
+		}		
+		var perPage = htmlOpts["rows"] * htmlOpts["columns"];		
+		var remainder = totalCount % perPage;
+		htmlOpts["numpages"] = Math.floor(totalCount / perPage);
+		if (remainder != 0) { 
+			htmlOpts["numpages"]++;
+		}
+
+		imLabels = fixDoubleArray(imLabels, remainder);
+		thumbwidths = fixDoubleArray(thumbwidths, remainder);
+		thumbheights = fixDoubleArray(thumbheights, remainder);
+		imageNames = fixDoubleArray(imageNames, remainder);
+		
+		// why does this run twice????
+		//alert ('imageNames ' + imageNames[0]);
+	}
+}
+		
+// config only works with tables
+if  (htmlOpts["config"] && !htmlOpts["tables"]) {
+	htmlOpts["config"] = 0;
+	}
+
 // Set number of display windows based on framestyle option
 var numWindows = 3; // default for framestyle 1
 if( htmlOpts["framestyle"] == 2 ||
-    htmlOpts["framestyle"] == 3 ) {
+    htmlOpts["framestyle"] == 3 || 
+    htmlOpts["framestyle"] == 4) {
   numWindows = 4;
 }
 
@@ -58,6 +122,62 @@ else if( numWindows == 4 ) {
   frameTargets["thumbview"]  = 'thumbview';
 }
 
+function emptyHTML()
+{
+	return '<HTML><HEAD></HEAD><BODY></BODY></HTML>';
+		
+}
+
+
+function drawWindows()
+{
+	var style = htmlOpts["framestyle"];
+
+  // Is README actually displayed?
+  if( ( htmlOpts["readmepresent"] == 1 && htmlOpts["numpages"] == 0 )
+      || htmlOpts["readmevisible"] == 1 ) {
+    var showReadme = 1;
+  } else {
+    var showReadme = 0;
+  }
+
+	if( style == 1 ) {  	
+		displayResult(frameTargets["dirview"], returnDirectoryHTML());
+		displayResult(frameTargets["navview"], returnNavHTML(0));
+		if (showReadme != 1) {
+			displayResult(frameTargets["imageview"], returnThumbNailsHTML());
+		}
+	} else if (style == 2) {
+		displayResult(frameTargets["dirview"], returnDirectoryHTML());
+		displayResult(frameTargets["thumbview"], returnThumbNailsHTML());
+		displayResult(frameTargets["navview"], returnNavHTML(0));
+		if (showReadme != 1) {
+			displayResult(frameTargets["imageview"], returnImageHTML(0));
+		}
+	} else if (style == 3) {
+		displayResult(frameTargets["dirview"], returnDirectoryHTML());
+		displayResult(frameTargets["thumbview"], returnThumbNailsHTML());
+		displayResult(frameTargets["navview"], returnNavHTML(0));
+		if (showReadme != 1) {
+			displayResult(frameTargets["imageview"], returnImageHTML(0));
+		}
+	} else if (style == 4) {
+		displayResult(frameTargets["thumbview"], returnThumbNailsHTML());
+		displayResult(frameTargets["navview"], returnNavHTML(0));
+		displayResult(frameTargets["dirview"], returnDirectoryHTML());
+		if (showReadme != 1) {
+			displayResult(frameTargets["imageview"], returnImageHTML(0));
+		}
+	}
+	
+}
+
+function displayResult(target, result)
+{	
+	var theFrame = open("", target);
+	theFrame.document.writeln(result);
+	theFrame.document.close();	
+}
 
 function returnFrameHTML()
 {
@@ -79,12 +199,12 @@ function returnFrameHTML()
     ' MARGINWIDTH=' + htmlOpts["framemarginwidth"] + ' MARGINHEIGHT=' + htmlOpts["framemarginheight"];
 
   // Javascript URLs to generate frame source
-  var srcDirectory	= ' SRC="javascript:parent.returnDirectoryHTML();"';
+  var srcDirectory	= ' SRC="javascript:parent.emptyHTML();"';
   var srcHidden         = ' SRC="' + htmlOpts["jspageindex"] + '"';
-  var srcImageView	= ' SRC="javascript:parent.returnImageHTML(0);"';
-  var srcNavView        = ' SRC="javascript:parent.returnNavHTML(0);"';
+  var srcImageView	= ' SRC="javascript:parent.emptyHTML();"';
+  var srcNavView        = ' SRC="javascript:parent.emptyHTML();"';
   var srcReadMe         = ' SRC="' + htmlOpts["readme"] + '"';
-  var srcThumbnails	= ' SRC="javascript:parent.returnThumbNailsHTML();"';
+  var srcThumbnails	= ' SRC="javascript:parent.emptyHTML();"';
 
   // frame names
   var nameDirView 	= ' NAME="' + frameTargets["dirview"] + '"';
@@ -96,8 +216,10 @@ function returnFrameHTML()
   var result = 
     '<HTML>\n' +
     '<HEAD>\n' +
-    '<TITLE>' + htmlOpts["title"] + '</TITLE>\n' +
-    '</HEAD>\n';
+    '<TITLE>' + htmlOpts["title"] + '</TITLE>\n';
+    if (htmlOpts["stylesheet"])
+     result += '<LINK REL="stylesheet" type="text/css" HREF="' + htmlOpts["stylesheet"] + '">\n';
+    result += '</HEAD>\n';
 
   if( style == 1 ) {
     // Three frame screen with directories listed in the
@@ -316,7 +438,7 @@ function returnNavThumbHTML()
 	result += '<FORM NAME="navcontrol">\n';
 	result += '<FONT SIZE=1>\n';
 	result += '<SELECT NAME="page"' +
-	    ' onChange="parent.webmagick.pageNumber=document.navcontrol.page.selectedIndex;parent.webmagick.displayThumbNails();">\n';
+	    ' onChange="parent.webmagick.pageNumber=document.navcontrol.page.selectedIndex;parent.webmagick.pageImageIndex=0;parent.webmagick.displayThumbNails();">\n';
 	for( pageNum = 0; pageNum < htmlOpts["numpages"]; ++pageNum ) {
 	    if( pageNum == pageNumber ) {
 		result += '<OPTION VALUE="' + pageNum + '" SELECTED>' + (pageNum + 1) + '</OPTION>\n';
@@ -359,18 +481,30 @@ function returnNavThumbHTML()
 function returnNavHTML()
 {
     // page header
-    var result =
-	'<HTML>\n' +
-	'<HEAD>\n' +
-	'</HEAD>\n' +
-	'<BODY\n' +
-	'    TEXT="' + htmlOpts["colorfore"] + '"\n' +
-	'    BGCOLOR="' + htmlOpts["colorback"] + '"\n' +
-	'    BACKGROUND="' + iconImageUrls["background"] + '"\n' +
-	'    LINK="' + htmlOpts["colorlink"] + '"\n' +
-	'    VLINK="' + htmlOpts["colorvlink"] + '"\n' +
-	'    ALINK="' + htmlOpts["coloralink"] + '"\n' +
-	'>\n';
+      var result =
+    '<HTML>\n' +
+    '<HEAD>\n' ;
+    if (htmlOpts["stylesheet"])
+    {
+     result += '<LINK REL="stylesheet" type="text/css" HREF="' + htmlOpts["stylesheet"] + '">\n';
+     result += '</HEAD>\n<BODY>\n';
+    }
+    else
+    {
+	    result += '</HEAD>\n' +
+	    '<BODY\n' +
+	    '    TEXT="' + htmlOpts["colorfore"] + '"\n' +
+	    '    BGCOLOR="' + htmlOpts["colorback"] + '"\n';
+	    if(iconImageUrls["background"]){
+	    	result +='    BACKGROUND="' + iconImageUrls["background"] + '"\n';
+	    	}
+	    result +='    LINK="' + htmlOpts["colorlink"] + '"\n' +
+	    '    VLINK="' + htmlOpts["colorvlink"] + '"\n' +
+	    '    ALINK="' + htmlOpts["coloralink"] + '"\n' +
+	    '>\n';
+     }
+
+ 
 
     //alert('returnNavHTML():  thumbNailMode=' + thumbNailMode);
 
@@ -472,57 +606,72 @@ imageExtensions['xwd']  = 'X Windows system window dump file (color)';
 // return HTML to display image with corresponding index
 function returnImageHTML(imageIndex)
 {
+	
   pageImageIndex = imageIndex;
 
-  var result =
+       var result =
     '<HTML>\n' +
-    '<HEAD>\n' +
-    '</HEAD>\n' +
-    '<BODY\n' +
-    '    TEXT="' + htmlOpts["colorfore"] + '"\n' +
-    '    BGCOLOR="' + htmlOpts["colorback"] + '"\n' +
-    '    BACKGROUND="' + iconImageUrls["background"] + '"\n' +
-    '    LINK="' + htmlOpts["colorlink"] + '"\n' +
-    '    VLINK="' + htmlOpts["colorvlink"] + '"\n' +
-    '    ALINK="' + htmlOpts["coloralink"] + '"\n' +
-    '>\n';
+    '<HEAD>\n' ;
+    if (htmlOpts["stylesheet"])
+    {
+     result += '<LINK REL="stylesheet" type="text/css" HREF="' + htmlOpts["stylesheet"] + '">\n';
+     result += '</HEAD>\n<BODY>\n';
+    }
+    else
+    {
+	    result += '</HEAD>\n' +
+	    '<BODY\n' +
+	    '    TEXT="' + htmlOpts["colorfore"] + '"\n' +
+	    '    BGCOLOR="' + htmlOpts["colorback"] + '"\n';
+	    if(iconImageUrls["background"]){
+	    	result +='    BACKGROUND="' + iconImageUrls["background"] + '"\n';
+	    	}
+	    result +='    LINK="' + htmlOpts["colorlink"] + '"\n' +
+	    '    VLINK="' + htmlOpts["colorvlink"] + '"\n' +
+	    '    ALINK="' + htmlOpts["coloralink"] + '"\n' +
+	    '>\n';
+     }
 
-  result += '  <H2 ALIGN="center">';
-  // if image title defined, then use it, otherwise, image name 
-  if( imageLabels[imageNames[pageNumber][imageIndex]] != null ) {
-    result += imageLabels[imageNames[pageNumber][imageIndex]];
-  } else {
-    result += imageNames[pageNumber][imageIndex];
-  }
-  result +=
-    '</H2>\n';
-
-  // Look for image file extension
-  var image = imageNames[pageNumber][imageIndex];
-  var delim = image.lastIndexOf('.');
-  var extension = '';
-  if ( delim > 0 ) {
-    extension = image.substring(delim + 1);
-    extension = extension.toLowerCase();
-  }
-
-  if( extension == 'gif' || extension == 'jpg' || extension == 'jpeg' ) {
-    // Browser supports displaying image in-line
-    result += '<P ALIGN="center"><IMG SRC="' + image + '"></P>\n';
-  }
-  else {
-    // Browser requires an external helper program
-    result +=
-      '<P>Your browser may not have the capability to view files of format "' + imageExtensions[extension]
-	+ '" as an in-line image. View image via <A HREF="' + image + '">a link</A> instead.</P>\n';
-  }
-  if(htmlOpts["anonymous"] == 0) {
-    result +=
-      '<BR><HR>\n' +
-      '<ADDRESS><CENTER>Produced by ' +
-      '<A HREF="http://www.simplesystems.org/WebMagick/">WebMagick</A> ' +
-      htmlOpts["version"] + '\nCopyright &copy; Bob Friesenhahn</CENTER></ADDRESS>\n';
-  }
+	// only do work if image exists
+	if (imageNames[pageNumber][imageIndex] != null) {
+	
+	  result += '  <H2 ALIGN="center">';
+	  // if image title defined, then use it, otherwise, image name 
+	  if( imageLabels[imageNames[pageNumber][imageIndex]] != null ) {
+	    result += imageLabels[imageNames[pageNumber][imageIndex]];
+	  } else {
+	    result += imageNames[pageNumber][imageIndex];
+	  }
+	  result +=
+	    '</H2>\n';
+	
+	  // Look for image file extension
+	  var image = imageNames[pageNumber][imageIndex];
+	  var delim = image.lastIndexOf('.');
+	  var extension = '';
+	  if ( delim > 0 ) {
+	    extension = image.substring(delim + 1);
+	    extension = extension.toLowerCase();
+	  }
+	
+	  if( extension == 'gif' || extension == 'jpg' || extension == 'jpeg' ) {
+	    // Browser supports displaying image in-line
+	    result += '<P ALIGN="center"><IMG SRC="' + image + '"></P>\n';
+	  }
+	  else {
+	    // Browser requires an external helper program
+	    result +=
+	      '<P>Your browser may not have the capability to view files of format "' + imageExtensions[extension]
+		+ '" as an in-line image. View image via <A HREF="' + image + '">a link</A> instead.</P>\n';
+	  }
+	  if(htmlOpts["anonymous"] == 0) {
+	    result +=
+	      '<BR><HR>\n' +
+	      '<ADDRESS><CENTER>Produced by ' +
+	      '<A HREF="http://webmagick.sourceforge.net/" target="_top">WebMagick</A> ' +
+	      htmlOpts["version"] + '\nCopyright &copy; Bob Friesenhahn</CENTER></ADDRESS>\n';
+	  }
+	}
   result +=
     '</BODY>\n' +
     '</HTML>\n';
@@ -536,8 +685,9 @@ function returnImageHTML(imageIndex)
 var imageViewWindow;
 function displayImage()
 {
-    var targeturl = 'javascript:parent.webmagick.returnImageHTML(' + pageImageIndex + ');';
-    imageViewWindow = window.open( targeturl, frameTargets["imageview"]);
+    //var targeturl = 'javascript:parent.webmagick.returnImageHTML(' + pageImageIndex + ');';
+    //imageViewWindow = window.open( targeturl, frameTargets["imageview"]);
+    displayResult(frameTargets["imageview"], returnImageHTML(pageImageIndex));
     
     if( htmlOpts["framestyle"] == 1) {
 	thumbNailMode = false;
@@ -545,8 +695,9 @@ function displayImage()
     
     // alert('displayImage():  thumbNailMode=' + thumbNailMode);
     
-    var targeturl = 'javascript:parent.webmagick.returnNavHTML();';
-    navViewWindow = window.open( targeturl, frameTargets["navview"]);
+    //var targeturl = 'javascript:parent.webmagick.returnNavHTML();';
+    //navViewWindow = window.open( targeturl, frameTargets["navview"]);
+    displayResult(frameTargets["navview"], returnNavHTML());
     
 }
 
@@ -620,13 +771,15 @@ var thumbViewWindow;
 var thumbNailMode=true;
 function displayThumbNails()
 {
-    var targeturl = 'javascript:parent.webmagick.returnThumbNailsHTML();';
-    thumbViewWindow = window.open( targeturl, frameTargets["thumbview"]);
+    //var targeturl = 'javascript:parent.webmagick.returnThumbNailsHTML();';
+    //thumbViewWindow = window.open( targeturl, frameTargets["thumbview"]);
+    displayResult(frameTargets["thumbview"], returnThumbNailsHTML());
 
     if ( htmlOpts["framestyle"] == 1 ) {
 	thumbNailMode = true
-	var targeturl = 'javascript:parent.webmagick.returnNavHTML();';
-	navViewWindow = window.open( targeturl, frameTargets["navview"]);
+	//var targeturl = 'javascript:parent.webmagick.returnNavHTML();';
+	//navViewWindow = window.open( targeturl, frameTargets["navview"]);
+	displayResult(frameTargets["navview"], returnNavHTML());
     } else {
 	thumbNailMode = true
 	displayImage();
@@ -638,18 +791,29 @@ function returnThumbNailsHTML()
 {
 
   // page header
-  var result =
+       var result =
     '<HTML>\n' +
-    '<HEAD>\n' +
-    '</HEAD>\n' +
-    '<BODY\n' +
-    '    TEXT="' + htmlOpts["colorfore"] + '"\n' +
-    '    BGCOLOR="' + htmlOpts["colorback"] + '"\n' +
-    '    BACKGROUND="' + iconImageUrls["background"] + '"\n' +
-    '    LINK="' + htmlOpts["colorlink"] + '"\n' +
-    '    VLINK="' + htmlOpts["colorvlink"] + '"\n' +
-    '    ALINK="' + htmlOpts["coloralink"] + '"\n' +
-    '>\n';
+    '<HEAD>\n' ;
+    if (htmlOpts["stylesheet"])
+    {
+     result += '<LINK REL="stylesheet" type="text/css" HREF="' + htmlOpts["stylesheet"] + '">\n';
+     result += '</HEAD>\n<BODY>\n';
+    }
+    else
+    {
+	    result += '</HEAD>\n' +
+	    '<BODY\n' +
+	    '    TEXT="' + htmlOpts["colorfore"] + '"\n' +
+	    '    BGCOLOR="' + htmlOpts["colorback"] + '"\n';
+	    if(iconImageUrls["background"]){
+	    	result +='    BACKGROUND="' + iconImageUrls["background"] + '"\n';
+	    	}
+	    result +='    LINK="' + htmlOpts["colorlink"] + '"\n' +
+	    '    VLINK="' + htmlOpts["colorvlink"] + '"\n' +
+	    '    ALINK="' + htmlOpts["coloralink"] + '"\n' +
+	    '>\n';
+     }
+
 
   // if no images then return empty page
   if( imageNames[pageNumber].length == 0 ) {
@@ -665,19 +829,60 @@ function returnThumbNailsHTML()
   }
 
   result +=
-    '<CENTER>\n' +
-    '<IMG SRC="' + montageImages[pageNumber] + '" ' +
-    montageImageSizes[pageNumber] +
-    ' USEMAP="#thumbnails" BORDER=0>' +
-    '<MAP NAME="thumbnails">\n';
-  for( imageNum = 0; imageNum < imageThumbCoords[pageNumber].length; ++imageNum ) {
-    result +=
-      '  <AREA HREF="javascript:parent.webmagick.displayImageByIndex(' + imageNum + ');"' +
-      ' SHAPE=RECT COORDS="' + imageThumbCoords[pageNumber][imageNum] + '">\n';
+    '<CENTER>\n';
+    
+  if ( htmlOpts["tables"] == 1) {  	
+  	result += '<TABLE WIDTH=90%>';  	
+  } else {
+	  result +=
+	    '<IMG SRC="' + montageImages[pageNumber] + '" ' +
+	    montageImageSizes[pageNumber] +
+	    ' USEMAP="#thumbnails" BORDER=0>' +
+	    '<MAP NAME="thumbnails">\n';
+  }
+  for( imageNum = 0; imageNum < imageNames[pageNumber].length; ++imageNum ) {
+  	if ( htmlOpts["tables"] == 1) {  	
+  		if (imageNum%htmlOpts["columns"] == 0) {
+  			result += '<TR ALIGN=CENTER VALIGN=TOP>';
+  		}
+  		result += '<TD><A HREF="javascript:parent.webmagick.displayImageByIndex(' + imageNum + ');"\n' +
+  		  ' onMouseOver="status=\'View ' + escapeJs(imageNames[pageNumber][imageNum]) + '\'; return true;" \n' +
+	      ' onMouseOut="status=\'\';">\n';
+  		result += '<IMG BORDER=0 SRC="';
+  		 // if thumb width is 0, then there must not be a cachefile (original image is small enough)
+  		 if ( thumbwidths[pageNumber][imageNum] != 0) {
+  		 	result += escapeURL(htmlOpts["cachedir"]) + '/';
+  		 }
+  		 result += escapeURL(imageNames[pageNumber][imageNum]) + '"';
+  		 if ( thumbwidths[pageNumber][imageNum] != 0) {
+  		 	result += ' HEIGHT=' + thumbheights[pageNumber][imageNum] + ' WIDTH= ' + thumbwidths[pageNumber][imageNum];
+ 		  }
+ 		 result += '></A><BR>\n'; 		  		  
+  		if (imLabels[pageNumber][imageNum]) {	
+  			result += '<FONT SIZE=-1>' + imLabels[pageNumber][imageNum] + '</FONT>\n';
+  		}
+  		result += '</TD>';
+  		if (imageNum%htmlOpts["columns"] == (htmlOpts["columns"] - 1)) {
+  			result += '</TR>';
+  		}
+  		 
+  		
+  	
+} else {
+	    result +=
+	      '  <AREA HREF="javascript:parent.webmagick.displayImageByIndex(' + imageNum + ');"\n' +	    
+	      ' onMouseOver="status=\'View ' + escapeJs(imageNames[pageNumber][imageNum]) + '\'; return true;"\n' +
+	      ' onMouseOut="status=\'\';"\n' +
+	      ' SHAPE=RECT COORDS="' + imageThumbCoords[pageNumber][imageNum] + '">\n';
+	}
   }
 
-  result +=
-    '</MAP>\n';
+  if ( htmlOpts["tables"] == 1) {
+  	result += '</TABLE>';  	
+  } else {
+	  result +=
+	    '</MAP>\n';
+   }
 
   result +=
     '</CENTER>\n' +
@@ -697,33 +902,104 @@ function returnThumbNailsHTML()
 var dirViewWindow;
 function displayDirectory()
 {
-  var targeturl = 'javascript:parent.webmagick.returnDirectoryHTML();';
-  dirViewWindow = window.open( targeturl, frameTargets["dirview"]);
+  displayResult(frameTargets["dirview"], returnDirectoryHTML());
+}
+
+var configWindow;
+function saveConfig()
+{	
+	newColumns = configWindow.document.aForm.columns.value;
+	newRows = configWindow.document.aForm.rows.value;
+	var newStyle;
+	for (var i = 0; i < configWindow.document.aForm.framestyle.length; i++)  {   
+		if (configWindow.document.aForm.framestyle[i].checked) {         
+			newStyle=configWindow.document.aForm.framestyle[i].value;
+		}   
+	}
+	
+	// TODO: error checking
+	
+	Set_Cookie('framestyle', newStyle, expires_date);
+	Set_Cookie('columns', newColumns, expires_date);
+	Set_Cookie('rows', newRows, expires_date);
+
+	configWindow.close();
+	parent.location.reload();
+	
+}
+
+// display the window to allow configuration changes
+function showConfig()
+{
+	configWindow = open('', 'wmConfig', 'toolbar=no,dependent=yes,menubar=no,status=no,resizable=yes,width=550,height=575');
+	configWindow.document.open();
+	result = '<HTML><HEAD><TITLE>Configure Gallery</TITLE></HEAD><BODY>';
+	
+	result += '<FORM NAME="aForm" ACTION="" ONSUBMIT="">';
+	
+	result += '<P>Columns: <INPUT WIDTH=5 TYPE="text" NAME="columns" VALUE="' + htmlOpts["columns"] + '">\n';
+	result += ' Rows: <INPUT WIDTH=5 TYPE="text" NAME="rows" VALUE="' + htmlOpts["rows"] + '">\n';
+	result += '<INPUT TYPE="reset" VALUE="Reset"><INPUT TYPE="button" VALUE="Save" onclick="window.opener.parent.webmagick.saveConfig()">';
+	result += '<BR><TABLE BORDER=1><TR><TD><INPUT TYPE="radio" VALUE="1" NAME="framestyle"';
+	if (htmlOpts["framestyle"] == 1) {
+		result += ' checked ';
+	}
+	result += '><IMG SRC="' + iconImageUrls["frame-style-1"] + '" ' + iconImageSizes["frame-style-1"] + '></TD>\n';
+	result += '<TD><INPUT TYPE="radio" VALUE="2" NAME="framestyle"';
+	if (htmlOpts["framestyle"] == 2) {
+		result += ' checked ';
+	}
+	result += '><IMG SRC="' + iconImageUrls["frame-style-2"] + '" ' + iconImageSizes["frame-style-2"] + '></TD></TR>\n';
+	result += '<TR><TD><INPUT TYPE="radio" VALUE="3" NAME="framestyle"';
+	if (htmlOpts["framestyle"] == 3) {
+		result += ' checked ';
+	}
+	result += '><IMG SRC="' + iconImageUrls["frame-style-3"] + '" ' + iconImageSizes["frame-style-3"] + '></TD>\n';
+	result += '<TD><INPUT TYPE="radio" VALUE="4" NAME="framestyle"';
+	if (htmlOpts["framestyle"] == 4) {
+		result += ' checked ';
+	}
+	result += '><IMG SRC="' + iconImageUrls["frame-style-4"] + '" ' + iconImageSizes["frame-style-4"] + '></TD></TR></TABLE>\n';
+	result += '</FORM></BODY></HTML>';
+	configWindow.document.write(result);
+	configWindow.document.close();
+	
 }
 
 // Navigate to directory
 function changeDirectory(dirname)
 {
   // Redisplay complete frameset
-  top.window.location = dirname + '/' + dirLinks[dirname];
+  top.window.location = escapeURL(dirname) + '/' + dirLinks[dirname];
 }
 
 // return HTML to display directory listing
 function returnDirectoryHTML()
 {
-  var result =
-    '<HEAD>\n' +
-    '<BASE TARGET="_top"\n' +
-    '</HEAD>\n' +
-    '<BODY\n' +
-    '    TEXT="' + htmlOpts["dircolorfore"] + '"\n' +
-    '    BGCOLOR="' + htmlOpts["dircolorback"] + '"\n' +
-    '    BACKGROUND="' + iconImageUrls["dirbackground"] + '"\n' +
-    '    LINK="' + htmlOpts["dircolorlink"] + '"\n' +
-    '    VLINK="' + htmlOpts["dircolorvlink"] + '"\n' +
-    '    ALINK="' + htmlOpts["dircoloralink"] + '"\n' +
-    '>\n' +
-    '<FONT SIZE=-1>\n';
+       var result =
+    '<HTML>\n' +
+    '<HEAD>\n' ;
+    if (htmlOpts["stylesheet"])
+    {
+     result += '<LINK REL="stylesheet" type="text/css" HREF="' + htmlOpts["stylesheet"] + '">\n';
+     result += '</HEAD>\n<BODY>\n';
+    }
+    else
+    {
+	    result += '</HEAD>\n' +
+	    '<BODY\n' +
+	    '    TEXT="' + htmlOpts["colorfore"] + '"\n' +
+	    '    BGCOLOR="' + htmlOpts["colorback"] + '"\n';
+	    if(iconImageUrls["background"]){
+	    	result +='    BACKGROUND="' + iconImageUrls["background"] + '"\n';
+	    	}
+	    result +='    LINK="' + htmlOpts["colorlink"] + '"\n' +
+	    '    VLINK="' + htmlOpts["colorvlink"] + '"\n' +
+	    '    ALINK="' + htmlOpts["coloralink"] + '"\n' +
+	    '>\n';
+     }
+
+    result +='<FONT SIZE=-1>\n';
   // up arrow
   result +=
     '<A HREF="javascript:parent.webmagick.changeDirectory(\'..\');"\n' +
@@ -732,6 +1008,17 @@ function returnDirectoryHTML()
     '  ><IMG SRC="' + iconImageUrls["up"] + '" ' + iconImageSizes["up"] +
     ' ALT="up one directory" BORDER=0></A>\n';
   
+  // config
+  if( htmlOpts["config"] && (htmlOpts["config"] != 0)) {
+    result +=
+      '<A HREF="javascript:parent.webmagick.showConfig();"\n' +
+      '  onMouseOver="status=\'configure your frames\'; return true;"\n' +
+      '  onMouseOut="status=\'\';"\n' +
+      '  ><IMG SRC="' + iconImageUrls["config"] + '" ' + iconImageSizes["config"] +
+      ' ALT="Config" BORDER=0></A>\n';
+  }
+  
+  
   // help
   if( htmlOpts["readmepresent"] != 0 ) {
     result +=
@@ -743,9 +1030,11 @@ function returnDirectoryHTML()
       ' ALT="ReadMe" BORDER=0></A>\n';
   }
   
+  
+  
   // list out directory URLs
   if( dirNames.length != 0 ) {
-    result += '<B>Directories</B><BR>\n';
+    result += '<BR><B>Directories</B><BR>\n';
     for( dirNum = 0; dirNum < dirNames.length; ++dirNum ) {
       
       // if we have directory title, then use it, else use directory name
@@ -755,9 +1044,8 @@ function returnDirectoryHTML()
       } else {
 	dirTitle = dirNames[dirNum];
       }
-      
       result +=
-	'<NOBR><A HREF="javascript:parent.webmagick.changeDirectory(\'' + dirNames[dirNum] + '\');"\n' +
+	'<NOBR><A HREF="javascript:parent.webmagick.changeDirectory(\'' + escapeJs(dirNames[dirNum]) + '\');"\n' +
 	'  onMouseOver="status=\'change to directory: ' + escapeJs(dirTitle) + '\'; return true;"\n' +
 	'  onMouseOut="status=\'\';"\n';
       result += '>';
@@ -766,7 +1054,7 @@ function returnDirectoryHTML()
     }
   }
   
-  result += '</BODY>\n' ;
+  result += '</BODY></HTML>\n' ;
   
   return result;
 }
@@ -814,6 +1102,8 @@ function goToIndex(number)
 function escapeJs(inString)
 {
   var outString = '';
+  //alert ('Inside escapeJs ' + inString);
+  //if (! defined(inString)) { return outString; }
   for(pos = 0; pos < inString.length; pos++) {
     c = inString.charAt(pos);
     if(c == '\'' || c == '"' || c == '\n') {
@@ -824,5 +1114,92 @@ function escapeJs(inString)
   return outString;
 }
 
+// Escape spaces in URLs
+function escapeURL(inString)
+{
+        var outString = '';
+        for (pos = 0; pos < inString.length; pos++)
+        {
+                c = inString.charAt(pos);
+                if (c == '%')
+                {
+                        outString += '%25';
+                }
+                else if (c == ' ')
+                {
+                        outString += '%20';
+                }
+                else if (c == '#')
+                {
+                	outString += '%23';
+                }
+                else
+                {
+                        outString += c;
+                }
+        }
+        return outString;
+}
+
+function Get_Cookie(name) {
+	
+    var start = document.cookie.indexOf(name+"=");
+    var len = start+name.length+1;
+    if ((!start) && (name != document.cookie.substring(0,name.length))) return null;
+    if (start == -1) return null;
+    var end = document.cookie.indexOf(";",len);
+    if (end == -1) end = document.cookie.length;
+    return unescape(document.cookie.substring(len,end));
+}
+
+function Set_Cookie(name,value,expires,domain,secure) {
+    document.cookie = name + "=" +escape(value) +
+        ( (expires) ? ";expires=" + expires.toGMTString() : "") +
+        ";path=" + htmlOpts["prefixpath"] + "/" + 
+        ( (domain) ? ";domain=" + domain : "") +
+        ( (secure) ? ";secure" : "");
+               
+}
+
+function Delete_Cookie(name,path,domain) {
+    if (Get_Cookie(name)) document.cookie = name + "=" +
+        ( (path) ? ";path=" + path : "") +
+        ( (domain) ? ";domain=" + domain : "") +
+        ";expires=Thu, 01-Jan-1970 00:00:01 GMT";
+}
+
+function fixDoubleArray(anArray, aRemainder) {
+	
+	var counter = 0;
+	var perPage = htmlOpts["rows"] * htmlOpts["columns"];
+	
+	// create new array
+	var tempArray = new Array(htmlOpts["numpages"]);
+	for (i = 0; i < htmlOpts["numpages"]; i++) {
+		if ((i == htmlOpts["numpages"] - 1) && (aRemainder != 0)) {
+			tempArray[i] = new Array(aRemainder);
+		} else {
+			tempArray[i] = new Array(perPage);
+		}		
+	}
+	
+	// handle empty directory
+	if (htmlOpts["numpages"] == 0) {
+		tempArray = new Array(1);
+		tempArray[0] = new Array(0);
+	}
+			
+	for (i = 0; i < anArray.length; i++) {
+		for (j = 0; j < anArray[i].length; j++) {			
+			tempArray[(Math.floor(counter / perPage))][(counter % perPage)] = anArray[i][j];			
+			counter++;
+		}		
+	}
+	
+	//alert ('tempArray ' + tempArray[0]);
+	//anArray = tempArray;	
+	return tempArray;
+}
+
 // indicate this file fully loaded
 var WebMagickLoaded = 1;
